import 'dart:typed_data';

import '../constants.dart';
import '../decoders/auth_message_decoders.dart';
import '../decoders/decoders.dart';
import '../enums.dart';
import '../types/odid_message.dart';
import 'odid_message_parser.dart';

class AuthMessageParser implements ODIDMessageParser {
  const AuthMessageParser();

  /// see comment above [AuthMessage] for explanation of the format
  @override
  AuthMessage? parse(Uint8List messageData) {
    final protocolVersion =
        decodeField(decodeProtocolVersion, messageData, 0, 1);
    final authType = AuthType.getByValue((messageData[1] & 0xF0) >> 4);

    if (protocolVersion == null || authType == null) return null;

    int? pageNumber = messageData[1] & 0x0F;
    int? lastPageIndex;
    int? authLength;
    DateTime? timestamp;
    int? authDataOffset;
    int? currMessageAuthLength;

    if (pageNumber == 0) {
      lastPageIndex = messageData[2] & 0x0F;
      authLength = messageData[3];
      timestamp = decodeField(decodeODIDEpochTimestamp, messageData, 4, 8);

      if (lastPageIndex >= maxAuthDataPages ||
          authLength > maxAuthPageZeroSize) {
        return null;
      }

      authDataOffset = 8;
      currMessageAuthLength = maxAuthPageZeroSize;
    } else {
      authDataOffset = 2;
      currMessageAuthLength = maxAuthPageNonZeroSize;
    }

    final authData = decodeField(
      decodeAuthData,
      messageData,
      authDataOffset,
      authDataOffset + currMessageAuthLength,
    );

    if (authData == null) return null;

    return AuthMessage(
      protocolVersion: protocolVersion,
      rawContent: messageData,
      authType: authType,
      authPageNumber: pageNumber,
      lastAuthPageIndex: lastPageIndex,
      authLength: authLength,
      timestamp: timestamp,
      authData: authData,
    );
  }
}
